// command.c
/*----------------------------------------------------------------------*/
/*! \file 
--------------------------------------------------------------------------

  	<h3> Command.C </h3>
  	Version	1.0<br>
  	Revision	$Revision: 1.4 $<br>
 	Copyright 	Thomas Wedemeyer, Email: avr@thomas-wedemeyer.de<br> 
  	Datum		$Date<br>

	<h1>Commandointerface for IO-Interface for the Linksys WRT54 
	with Atmel AVR ATMega8</h1>  

/*----------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <inttypes.h>
#include <avr/pgmspace.h>
#include <avr/delay.h>
#include "uart.h"
#include "command.h"
#include "lcd.h"
#include "adc.h"



extern const char ok_msg[] PROGMEM;
extern const char error_msg[] PROGMEM;

/*----------------------------------------------------------------------*/
//! Simple Command-Parser
/*----------------------------------------------------------------------*/
void command()
{
	uint8_t *ptr;

	ptr = uart_getrxptr();	// Get Adress of the Rx-Buffer

	// Message for the lcd ?!
	while(*ptr != '\e')	// No Escape-Character
	{
		if(*ptr == 0)	// End of string
		{
			uart_clearbuf();
			_writePString(ok_msg);
			return;
		}

		write_lcd(0, *ptr);
		_delay_us(40);
		ptr++;
	}

	// Got a Escape Character
	ptr++;

	switch(*ptr)
	{
		case 'A':	// Analog Input
		case 'a':	
					get_analogvalue(ptr);
					break;

		case 'C':	// Clear LCD
		case 'c':	
					clr_lcd();
					uart_clearbuf();
					_writePString(ok_msg);
					break;

		case 'D':	// Write displaycommand
		case 'd':
					write_displaycmd(ptr);
					break;

		case 'i':
		case 'I': 	// Input
					get_input(ptr);
					break;


		case 'P':	// Print LCD
		case 'p':
					print2lcd(ptr);
					break;	

		case 'R':	// Relais
		case 'r':	
					set_relais(ptr);
					break;

		case 'W':	//Write I2C
					//write_bus(ptr);
					break;
		case 'B':	// Read I2C
					//read_bus(ptr);
					break;
		default:
			_writePString(error_msg);
			uart_clearbuf();	
			return;
	}
}


/*----------------------------------------------------------------------*/
//! Read digital Input
//! @param	ptr	Pointer to command-string
/*----------------------------------------------------------------------*/
void get_input( uint8_t *ptr)
{
	const uint8_t mask[4] = { 0x01,0x02,0x04,0x08};
	int8_t input;
	uint8_t tmp=0;

	ptr++;
	input = *ptr -'0';
	
	uart_clearbuf();

	if((input < 0) | (input > 3))
	{
		_writePString(error_msg);
		return;
	}

	//printf_P(PSTR("PIND=0x%02x PINC=0x%02x Input=%d\r\n"),PIND,PINC,input);

	tmp = (((PIND & 0x0C) >> 2) | ((PINC & 0x03) <<2)) & mask[input];

	if(tmp)
		_writePString (PSTR("I1\r\n"));
	else
		_writePString (PSTR("I0\r\n"));

	_writePString(ok_msg);
}


/*----------------------------------------------------------------------*/
//! Set Relais
//! @param	ptr	Pointer to command-string
/*----------------------------------------------------------------------*/
void set_relais( uint8_t *ptr)
{
	const uint8_t mask[4] = { 0x10,0x20,0x40,0x80};
	int8_t	relais;
	int8_t state;

	ptr++;
	relais = *ptr -'0';
	ptr++;
	state = *ptr -'0';
	
	uart_clearbuf();

	if((relais < 1) | (relais > 4) | 	
		(state < 0) | (state > 1))
	{
		_writePString(error_msg);
		return;
	}

	relais--;

	if(state)
		PORTD |= mask[relais];
	else
		PORTD &= ~mask[relais];

	_writePString(ok_msg);
}


/*----------------------------------------------------------------------*/
//! Print String on the LCD at a defined position
//! @param	ptr	Pointer to command-string
/*----------------------------------------------------------------------*/
void print2lcd(uint8_t *ptr)
{
	uint8_t		pos[2];
	uint8_t 	tmp;
	uint8_t		i;

	for( i = 0; i < 2; i++)
	{
		tmp = 0;
		ptr++;
		while(*ptr != ',')	// ASCII 2 Int conversation
		{
			if((*ptr < '0') | (*ptr > '9') | (*ptr == 0))	// No Number or End of String
			{
				uart_clearbuf();			// Return Error
				_writePString(error_msg);
				return;
			}

			tmp *= 10;
			tmp += *ptr - '0';
			ptr++;
		}

		pos[i] = (uint8_t)tmp;

	}

	ptr++;

	print_lcd(pos[0], pos[1],(uint8_t *) ptr);
	uart_clearbuf();			// Return Ok
	_writePString(ok_msg);
}

/*----------------------------------------------------------------------*/
//! Write a commandcode to the display
//! @brief Commandsequence is transmitted as dezimal numbers separated by ","  
//! @param	ptr	Pointer to command-string
/*----------------------------------------------------------------------*/
void write_displaycmd(uint8_t *ptr)
{
	uint8_t 	tmp;

	while(*ptr != 0)
	{
		tmp = 0;
		ptr++;

		while((*ptr != ',') & (*ptr != 0))	// ASCII 2 Int conversation
		{
			if((*ptr < '0') | (*ptr > '9'))	// No Number
			{
				uart_clearbuf();			// Return Error
				_writePString(error_msg);
				return;
			}

			tmp *= 10;
			tmp += *ptr - '0';
			ptr++;
		}

		write_lcd( 1, tmp);
		_delay_ms(1);
	}

	uart_clearbuf();			// Return Ok
	_writePString(ok_msg);
}

/*----------------------------------------------------------------------*/
//! Get Values from the analog port
//! @param	ptr	Pointer to command-string
/*----------------------------------------------------------------------*/
void get_analogvalue( uint8_t *ptr)
{
	int8_t	channel;
	uint16_t	value;
	uint8_t buffer[5];


	ptr++;
	channel = *ptr -'0';

	if((channel >= 0) & (channel < 4))
	{
		value = meansample_adc(channel);	
		itoa(value, buffer, 10);
		
		_writePString (PSTR("A"));		
		_writeString(buffer);
		_writePString (PSTR("\r\n"));
	
		uart_clearbuf();
		return;
	}
	
	uart_clearbuf();
	_writePString(error_msg);

}


