// lcd.c
/*----------------------------------------------------------------------*/
/*! \file 
/*----------------------------------------------------------------------*/
/*
  	<h3> lcd.C </h3>
  	Version	1.0<br>
  	Revision	$Revision: 1.4 $<br>
 	Copyright 	Thomas Wedemeyer, Email: avr@thomas-wedemeyer.de<br> 
  	Datum		$Date<br>

	<h1>IO-Interface for The Linksys WRT54 with Atmel AVR ATMega8</h1>  

	Controlfunctions for the HD44780-LCD in 4-Bit Mode
/*----------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/delay.h>
#include "pin.h"
#include "lcd.h"


/*----------------------------------------------------------------------*/
//! @brief Write a byte to the lcd
//! @param reg	1 = register access , 0 = display data
/*----------------------------------------------------------------------*/
void write_lcd(uint8_t reg, uint8_t data)
{
	// Registerzugriff ?
	if (reg==1)
	{
		LCDPORT &= ~LCD_RS;	// Register access
	}
	else
	{
		LCDPORT |= LCD_RS; // Display Data
	}
	
	// Write MSB
	LCDPORT &= 0xF0;	
	LCDPORT |= (data & 0xF0) >> 4;

	// E-Strobe
	LCDPORT |= LCD_E;	
	asm("nop");
	asm("nop");
	LCDPORT &= ~LCD_E;

	// Write LSB
	LCDPORT &= 0xF0;	
	LCDPORT |= (data & 0x0F);

	// E-Strobe
	LCDPORT |= LCD_E;	
	asm("nop");
	asm("nop");
	LCDPORT &= ~LCD_E;
}

/*----------------------------------------------------------------------*/
//! @brief Init of the lcd in 4-Bit Mode
/*----------------------------------------------------------------------*/
void lcd_init()
{
	uint8_t i;

	_delay_ms(10);

	for(i = 0 ; i <3; i++)
	{
		LCDPORT &= ~LCD_RS;	// Register access

		// Write MSB
		LCDPORT &= 0xF0;	
		LCDPORT |= 0x03;

		// E-Strobe
		LCDPORT |= LCD_E;	
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		LCDPORT &= ~LCD_E;

		_delay_ms(5);
	}

	LCDPORT &= ~LCD_RS;	// Register access

	// Write MSB
	LCDPORT &= 0xF0;		// 4.Bit-Mode	
	LCDPORT |= 0x02;

	// E-Strobe
	LCDPORT |= LCD_E;	
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	LCDPORT &= ~LCD_E;

	_delay_ms(5);
		
	write_lcd(1,0x20 | LCD_LINEMD);	// Select Linemode
	_delay_ms(1);
	
	write_lcd(1, 0x06);		// Cursordirektion , Display on
	_delay_ms(1);

	write_lcd(1, 0x08 | 0x04 | LCD_CURSOR | LCD_BLINKING );// Cursorsetup
	_delay_ms(1);
			
	write_lcd(1,0x01);	// Display lschen
	_delay_ms(2);

	write_lcd(1,0x02);	// Cursor Home-Position
	_delay_ms(2);


}


/*----------------------------------------------------------------------*/
//! @brief Outputs a String at a defined position
//! @param	x	x-Pos in display row
//! @param	y	row-nr.
//! @param	*string	Pointer to String
/*----------------------------------------------------------------------*/
void print_lcd(int8_t x, int8_t y, uint8_t *string)
{
	uint8_t 	position=0;

	
	// Check position
	if ((x<0) | (x > (LCD_CHAR_PER_LINE-1)))	x = 0;
		
	if ((y<0) | (y > (LCD_LINES-1))) 	y = 0;

	
	// Berechnen der Display-Position
	switch(y)					
	{
		case 0: position = LCD_ROW0_BASE;						// Offset-Zeile 1
				break;
		case 1:	position = LCD_ROW1_BASE;						// Offset-Zeile 2
				break;
		case 2:	position = LCD_ROW0_BASE + LCD_CHAR_PER_LINE;	// Offset-Zeile 3
				break;
		case 3:	position = LCD_ROW1_BASE + LCD_CHAR_PER_LINE;	// Offset-Zeile 4
				break;
	}

	position=position+x;			// Berechnung Position
	position=position | 0x80;		// Mit Befehlscodierung verodern
	
	write_lcd(1,position);	// Cursor an Position setzen
	_delay_us(40);			

	while( *string != 0)
	{
		write_lcd(0,(uint8_t) *string);	// Zeichen ausgeben
		_delay_us(40);
					
		string++;				// Zeichenpointer weiterdrehen
		x++;					// Zeichenposition weiterdrehen
			
		if (x > (LCD_CHAR_PER_LINE-1))	// Wenn Auerhalb der Zeile
		{
			y++;
		
			if (y<4)
			{
				x=0;
				// Berechnen der Display-Position
				switch(y)					
				{
					case 0: position = LCD_ROW0_BASE;						// Offset-Zeile 1
							break;
					case 1:	position = LCD_ROW1_BASE;						// Offset-Zeile 2
							break;
					case 2:	position = LCD_ROW0_BASE + LCD_CHAR_PER_LINE;	// Offset-Zeile 3
							break;
					case 3:	position = LCD_ROW1_BASE + LCD_CHAR_PER_LINE;	// Offset-Zeile 4
							break;
				}
				position=position | 0x80;		// Mit Befehlscodierung verodern

				write_lcd(1,position);	// Cursor an Position setzen			
				_delay_us(40);
			}		
			else
			{
				return;
			}
		}

	};	
}	

/*------------------------------------------------------------------------
	Lschen des Displays
/*----------------------------------------------------------------------*/
void clr_lcd()
{	
	write_lcd(1,0x01);	// Display lschen
	_delay_ms(2);
	
	write_lcd(1,0x02);	// Cursor Home-Position
	_delay_ms(2);		
}



