// uart.c
/*----------------------------------------------------------------------*/
/*! \file 
--------------------------------------------------------------------------

  	<h3> Pin.h </h3>
  	Version	1.0<br>
  	Revision	$Revision: 1.4 $<br>
 	Copyright 	Thomas Wedemeyer, Email: avr@thomas-wedemeyer.de<br> 
  	Datum		$Date<br>

	<h1>IO-Interface for The Linksys WRT54 with Atmel AVR ATMega8</h1>  

	

/*----------------------------------------------------------------------*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <stdio.h>
#include "pin.h"
#include "uart.h"
#include "define.h"

static struct uart_receive rx;

/*----------------------------------------------------------------------*/
//! Function: Sending a single Character
//! @param	c	Character to send
//! @return	Always zero
/*----------------------------------------------------------------------*/
int _writeChar (char c)
{
    while (!(UCSRA & (1<<UDRE))) {} 
        UDR = c;
	return 0; 
}

/*----------------------------------------------------------------------*/
//! Send a string via the uart
//! @param	string	Pointer to the string
//! return 	always zero
/*----------------------------------------------------------------------*/
int8_t _writeString (uint8_t *string) 
{ 
     while (*string) 
         _writeChar (*string++); 
	return 0;
} 

/*----------------------------------------------------------------------*/
//! Send a constant string (from flash) via the uart
//! @param	string	Pointer to the string
//! return 	always zero
/*----------------------------------------------------------------------*/
int8_t _writePString (PGM_P string) 
{ 
	uint8_t	tmp;
//	uint16_t adr;

	//adr = string;

	//printf_P(PSTR("Zeiger %d Zeichen: %d \r\n"),adr,pgm_read_byte (string));

	while(1)
	{
		tmp = pgm_read_byte (string);
		if(tmp == '\0')
			return 0;	

    	_writeChar(tmp);
		string++;
	}		 
} 


/*----------------------------------------------------------------------*/
//! Interrupt Service Routine UART
/*----------------------------------------------------------------------*/
SIGNAL (SIG_UART_RECV)
{
	uint8_t	tmp;

	tmp = UDR;

	if(tmp == '\n')	// Filter New-Line
		return;

	if(tmp == '\r')	// <cr> received?
	{
		rx.buffer[rx.index] = 0;	// terminate String
		rx.index++;
		rx.new_data = 1;
		return;
	}
		
	rx.buffer[rx.index] = tmp;
	rx.index++;

	rx.index &= RXBUFFERMAX;
}


/*----------------------------------------------------------------------*/
//! Check New-Data-Flag
//! @return 0 = No new Data, 1 = New Data 
/*----------------------------------------------------------------------*/
uint8_t uart_newdata()
{
	return (rx.new_data);
}

 
/*----------------------------------------------------------------------*/
//! GetPointer to the RX-Buffer
//! @return pointer to the buffer 
/*----------------------------------------------------------------------*/
uint8_t *uart_getrxptr()
{
	return((uint8_t *)rx.buffer);
}

/*----------------------------------------------------------------------*/
//! Clear RX-Buffer
/*----------------------------------------------------------------------*/
void uart_clearbuf()
{
	rx.index = 0;
	rx.new_data = 0;
}

/*----------------------------------------------------------------------*/
//! Copy received data to a new buffer
//! @param	ptr Pointer to new_buffer 
//! @return Number of bytes copied  
/*----------------------------------------------------------------------*/
uint8_t uart_get(uint8_t *ptr)
{
	uint8_t i;

	for(i = 0; i < rx.index; i++)
	{
		*ptr = rx.buffer[rx.index];
		ptr++;
	}
	uart_clearbuf();
	return i;
}

	

/*----------------------------------------------------------------------*/
//! Init the UART
/*----------------------------------------------------------------------*/
void init_uart()
{
	uint16_t ubrr;

	ubrr = (uint16_t)USART_BAUD_SELECT;	// Baurate setzen
	UBRRH = (uint8_t)(ubrr >>8);
    UBRRL = (uint8_t) ubrr;  

    UCSRB = (1<<RXCIE) | (1<<TXEN) | (1<<RXEN);		// TX & Rx aktivieren
    UCSRC = (1 << URSEL) | (1<<UCSZ1)|(1<<UCSZ0); 	// 8-Bit Nachrichtenlnge

	#ifdef INIT_FOR_PRINTF
	fdevopen(_writeChar, NULL, 0);		// Funktionszeiger fr Printf-Ausgaben zuweisen
	#endif
}



